/* fileinfo.c -- az lstat() altal visszaadott informaciok 
   megjelenitese */

#include <errno.h>
#include <stdio.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/sysmacros.h>
#include <sys/types.h>
#include <time.h>
#include <unistd.h>

/* A szoveges ido leiras maximalis hossza. */
#define TIME_STRING_BUF 50

/* Az ido informaciot szovegge konvertaljuk. */
char* timeString(time_t t, char* buf) 
{
  struct tm * local;

  local = localtime(&t);
  strftime(buf, TIME_STRING_BUF, "%c", local);

  return buf;
}

/* Minden informaciot megjelenitunk a parameterkent kapott 
   allomanyrol. */
int statFile(char* file)
{
  struct stat statbuf; 
  char timeBuf[TIME_STRING_BUF];

  if(lstat(file, &statbuf)) 
  {
    fprintf(stderr, "A(z) %s allomany informacioi nem elerhetoek: %s\n",
      file, strerror(errno));
    return 1;
  }

  printf("Fajlnev   : %s\n", file);
  printf("Eszkoz    : major %d/minor %d  Inode  : %ld\n",
         major(statbuf.st_dev), minor(statbuf.st_dev), statbuf.st_ino);
  printf("Meret     : %-10ld       Tipus  : %07o  "
         "Jogok  : %05o\n", statbuf.st_size, 
         statbuf.st_mode & S_IFMT, statbuf.st_mode & ~(S_IFMT));
  printf("Tulajdonos: %d              Csoport: %d"
         "      Linkek#: %u\n", 
         statbuf.st_uid, statbuf.st_gid, (unsigned int)statbuf.st_nlink);
  printf("Letrehozas ideje: %s\n", timeString(statbuf.st_ctime, timeBuf));
  printf("Modositas  ideje: %s\n", timeString(statbuf.st_mtime, timeBuf));
  printf("Hozzaferes ideje: %s\n", timeString(statbuf.st_atime, timeBuf));

  return 0;
}

int main(int argc, char* argv[])
{
  int i;
  int res=0;

  /* Minden parancssori argumentumot fajlnevnek tekintunk es meghivjuk
     a statFile() fuggvenyt. */
  for(i=1; i < argc; i++)
  {
    /* Ha a statFile() egyszer is hibaval ter vissza akkor a res erteke
       hibat fog jelezni. */
    res |= statFile(argv[i]);

    /* Az utolso allomany kivetelevel mindig kiirunk egy ujsort. */
    if((argc - i) > 1) printf("\n");
  }

  return res;
}
