/* mknod.c - a szokasos /sbin/mknod parancs megvalositasa */

/* Egy eszkoz vagy pipe specialis allomanyt hoz letre.
   Tovabbi informaciok az mknod(1) manualban talalhatoak. */

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/sysmacros.h>
#include <unistd.h>

int main(int argc, char* argv[])
{
  /* Allomany neve. */
  char* path;
  /* Az allomany jogai. */
  int mode = 0666;
  /* Az eszkoz allomany major es minor szama. */
  int major = 0, minor = 0;
  /* Segedvaltozo. */
  char* end;
  /* A szukseges argumentumok szama + 1. */
  int args;

  /* Minimum 2 parameterre van szuksegunk. */
  if(argc < 3)
  {
    fprintf(stderr, "Hasznalat: %s <path> [b|c|u|p] <major> <minor>\n", argv[0]);
    return 1;
  }

  path = argv[1];

  /* A masodik parameter adja meg milyen specialis allomanyt kell
    letrehoznunk. */
  if(strcmp(argv[2], "p") == 0)
  {
    mode |= S_IFIFO;
    args = 3;
  }
  else if(strcmp(argv[2], "b") == 0)
  {
    mode |= S_IFBLK;
    args = 5;
  }
  else if((strcmp(argv[2], "c") == 0) || (strcmp(argv[2], "u") == 0))
  {
    mode |= S_IFCHR;
    args = 5;
  }
  else
  {
    fprintf(stderr, "Ismeretlen tipus: %s\n", argv[2]);
    return 1;
  } 

  /* Most mar tudjuk a szukseges argumentum szamot, igy megvizsgalhatjuk
     hogy megfelelo-e. */
  if(argc != args)
  {
    fprintf(stderr, "Hasznalat: %s <path> [b|c|u|p] <major> <minor>\n", argv[0]);
    return 1;
  }

  if(args == 5)
  {
    /* Ha a parameterek szama 4, akkor a major es minor szamokat be kell
       olvasnunk. */
    major = strtol(argv[3], &end, 0);
    if(*end)
    {
      fprintf(stderr, "Rossz major szam: %s\n", argv[3]);
      return 1;
    }

    minor = strtol(argv[4], &end, 0);
    if(*end)
    {
      fprintf(stderr, "Rossz minor szam %s\n", argv[4]);
      return 1;
    }
  }

  /* Ha pipe-ot hozunk letre, akkor az utolso parametert az mknod()
     figyelmen kivul hagyja. */
  if(mknod(path, mode, makedev(major, minor)))
  {
    fprintf(stderr, "Az mknod() hibaval tert vissza: %s\n", strerror(errno));
    return 1;
  }

  return 0;
}
